from discord.ext import commands
from discord import app_commands
import discord

from ..utils import role_panel_function as Func

class RolePanelLibRolePanelContextMenuCog(commands.Cog):
    def __init__(self, bot: commands.Bot):
        self.bot: commands.Bot = bot
        self.menu = app_commands.ContextMenu(
            name="役職パネル選択",
            callback=self.role_panel_cm,
            guild_ids=None
        )
        self.bot.tree.remove_command(self.menu.name, type=self.menu.type)
        self.bot.tree.add_command(self.menu)
        self.rp_fix = app_commands.ContextMenu(
            name="役職パネル修復",
            callback=self.role_panel_fix,
            guild_ids=None
        )
        self.bot.tree.remove_command(self.rp_fix.name, type=self.rp_fix.type)
        self.bot.tree.add_command(self.rp_fix)

    async def cog_unload(self) -> None:
        self.bot.tree.remove_command(self.menu.name, type=self.menu.type)
        self.bot.tree.remove_command(self.rp_fix.name, type=self.rp_fix.type)

    # テストコマンド
    @commands.Cog.listener()
    async def on_ready(self):
        print(f'load context menu cog: {self.__class__.__name__}')
        super().__init__()  # this is now required in this context.

    async def role_panel_cm(
        self, 
        interaction: discord.Interaction,
        message: discord.Message
        ):
        if not interaction.user.guild_permissions.administrator:
            await interaction.response.send_message("このコマンドは管理者のみ実行可能です。", ephemeral=True)
            return
        if message.author.id != self.bot.user.id:
            await interaction.response.send_message("このコマンドはBotが送信したメッセージにのみ実行可能です。", ephemeral=True)
            return
        Func.add_select_role_panel(interaction.user.id, message.id)
        await interaction.response.send_message(f"役職パネルを選択\n{message.jump_url}", ephemeral=True)

    async def role_panel_fix(
        self, 
        interaction: discord.Interaction,
        message: discord.Message
        ):
        if not interaction.user.guild_permissions.administrator:
            await interaction.response.send_message("このコマンドは管理者のみ実行可能です。", ephemeral=True)
            return
        if message.author.id != self.bot.user.id:
            await interaction.response.send_message("このコマンドはBotが送信したメッセージにのみ実行可能です。", ephemeral=True)
            return
        await interaction.response.defer(thinking=True, ephemeral=True)
        await Func.fix_select_role_panel(interaction, message)

async def setup(bot: commands.Bot):
    await bot.add_cog(RolePanelLibRolePanelContextMenuCog(bot))


import discord
from discord.ext import commands
import traceback

from ...utils import role_panel_function as Func

class RolePanelLibNormalSelectEvent():
    def __init__(self, bot: commands.Bot):
        self.bot: commands.Bot = bot

    async def call(self, interaction: discord.Interaction):
        try:
            custom_id: str = interaction.data["custom_id"] #interaction.dataからcustom_idを取り出す
            if custom_id == f"{Func.get_custom_id()}edit_select":
                value = interaction.data["values"][0]
                if interaction.user.id not in Func.select_role_panel:
                    await interaction.response.edit_message("役職パネルを選択してください。", view=None)
                    return
                message_id = Func.select_role_panel[interaction.user.id]
                message = await interaction.channel.fetch_message(message_id)
                if message == None:
                    await interaction.response.edit_message("役職パネルのメッセージが見つかりません。", view=None)
                    return
                embed = message.embeds[0]
                if embed == None:
                    await interaction.response.edit_message("役職パネルが見つかりません。", view=None)
                    return
                view: discord.ui.View = discord.ui.View()
                for text in embed.fields[0].value.split("\n"):
                    if ":<@&" in text:
                        emoji_text: str = text.split(":<@&")[0]
                        role_id: int = int(text.split(":<@&")[1].split(">")[0])
                        role: discord.Role = interaction.guild.get_role(role_id)
                        if role != None:
                            button: discord.ui.Button = discord.ui.Button(label=emoji_text, custom_id=f"{Func.get_custom_id()}{emoji_text}_type_{value}", style=Func.get_button_style())
                            view.add_item(button)
                await message.edit(view=view)
                await interaction.response.edit_message(content="役職パネルを更新しました。\n「許可」→「禁止」などの変更時に既存のユーザーが取得しているロールについては各自で対応してください。", view=None)
        except Exception:
            traceback.print_exc()



import discord
from discord.ext import commands
import traceback

from ...utils import role_panel_function as Func

class RolePanelLibRolePanelButtonEvent():
    def __init__(self, bot: commands.Bot):
        self.bot: commands.Bot = bot

    async def call(self, inter: discord.Interaction):
        try:
            custom_id: str = inter.data["custom_id"] #interaction.dataからcustom_idを取り出す
            print("ready role panel event")
            if custom_id.startswith(Func.get_custom_id()):
                print("actions role panel event")
                await inter.response.defer(ephemeral=True, thinking=True)
                role_tag: str = custom_id.split(Func.get_custom_id())[1]
                new_version: bool = False
                type_text: str = None
                if "_type_" in role_tag:
                    new_version = True
                    type_text = role_tag.split("_type_")[1]
                    role_tag = role_tag.split("_type_")[0]
                tag_reactions = ['🇦', '🇧', '🇨', '🇩', '🇪', '🇫', '🇬', '🇭', '🇮', '🇯', '🇰', '🇱', '🇲', '🇳', '🇴', '🇵', '🇶', '🇷', '🇸', '🇹', '🇺', '🇻', '🇼', '🇽', '🇾', '🇿']
                if role_tag in tag_reactions:
                    tag_reaction = role_tag
                else:
                    # role_tag（'a' ~ 'w'）をリストのインデックス（0 ~ 22）に変換
                    index = ord(role_tag) - ord('a')  # 'a' = 0, 'b' = 1, ..., 'w' = 22
                    if 0 <= index < len(tag_reactions):  # 範囲内チェック
                        tag_reaction = tag_reactions[index]
                    else:
                        tag_reaction = None  # 範囲外なら None やデフォルト値を設定
                if len(inter.message.embeds) == 0:
                    print("embedがありません。")
                    return
                embed: discord.Embed = inter.message.embeds[0]
                single_role_type: int = 0
                if new_version:
                    if type_text == "single":
                        single_role_type = 1
                    elif type_text == "special":
                        single_role_type = 2
                    elif type_text == "add_only":
                        single_role_type = 3
                    elif type_text == "remove_only":
                        single_role_type = 4
                else:
                    if embed.fields[1].value == "禁止" or embed.fields[1].value == "重複を許可しない":
                        single_role_type = 1
                    elif embed.fields[1].value == "特殊":
                        single_role_type = 2
                    elif embed.fields[1].value == "付与専用":
                        single_role_type = 3
                    elif embed.fields[1].value == "取り外し専用":
                        single_role_type = 4
                role_list_text: str = embed.fields[0].value
                role_id: int = int(role_list_text.split(f"{tag_reaction}:<@&")[1].split(">")[0])
                role: discord.Role = inter.guild.get_role(role_id)
                if role == None:
                    await inter.followup.send(content="役職が見つかりませんでした。", ephemeral=True)
                    return
                add_role: bool = True
                if single_role_type == 1 or single_role_type == 2:
                    for temp_text in role_list_text.split("\n"):
                        temp_role_id: int = int(temp_text.split(f":<@&")[1].split(">")[0])
                        temp_role: discord.Role = inter.guild.get_role(temp_role_id)
                        if temp_role != None:
                            if single_role_type == 1:
                                await inter.user.remove_roles(temp_role)
                            else: # 特殊
                                if temp_role in inter.user.roles and temp_role != role:
                                    await inter.followup.send(content=f"{role.mention} は付与できません。{role.mention} を取得したい場合は {temp_role.mention} を取り外してから付与してください。", ephemeral=True)
                                    single_role_type = -2
                                    break
                    if single_role_type == -1:
                        if role in inter.user.roles:
                            await inter.user.remove_roles(role)
                            add_role = False
                        else:
                            await inter.user.add_roles(role)
                        await inter.followup.send(content=f"{role.mention} を{'追加' if add_role else '削除'}しました。", ephemeral=True)
                    elif single_role_type == 2:
                        if role in inter.user.roles:
                            await inter.user.remove_roles(role)
                            add_role = False
                        else:
                            await inter.user.add_roles(role)
                        await inter.followup.send(content=f"{role.mention} を{'追加' if add_role else '削除'}しました。", ephemeral=True)
                    elif single_role_type == 1:
                        await inter.user.add_roles(role)
                        await inter.followup.send(content=f"{role.mention} を追加しました。", ephemeral=True)
                else:
                    if single_role_type != 1:
                        if single_role_type == 3:
                            if role in inter.user.roles:
                                await inter.followup.send(content=f"{role.mention} は付与専用のため、削除できません。", ephemeral=True)
                                return
                            await inter.user.add_roles(role)
                            add_role = True
                        elif single_role_type == 4:
                            if role not in inter.user.roles:
                                await inter.followup.send(content=f"{role.mention} は取り外し専用のため、追加できません。", ephemeral=True)
                                return
                            await inter.user.remove_roles(role)
                            add_role = False
                        else:
                            if role in inter.user.roles:
                                await inter.user.remove_roles(role)
                                add_role = False
                            else:
                                await inter.user.add_roles(role)
                    await inter.followup.send(content=f"{role.mention} を{'追加' if add_role else '削除'}しました。", ephemeral=True)
        except Exception:
            traceback.print_exc()


from discord.ext import commands
from discord import app_commands
import discord
import traceback
from typing import List

from ..utils import role_panel_function as Func

class RolePanelLibRolePanelEditModal(discord.ui.Modal, title="役職パネル編集"):
    def __init__(self, bot: commands.Bot, type: str, value: str = None):
        super().__init__()
        self.bot = bot
        self.type = type
        if type == "タイトル":
            self.input = discord.ui.TextInput(
                label="タイトル",
                required=True,
                style=discord.TextStyle.short,
                placeholder="新しいタイトルを入力してください。",
                default=value
            )
            self.add_item(self.input)
        elif type == "説明":
            self.input = discord.ui.TextInput(
                label="説明",
                required=True,
                style=discord.TextStyle.long,
                placeholder="新しい説明を入力してください。",
                default=value
            )
            self.add_item(self.input)
        
    async def on_submit(self, interaction: discord.Interaction):
        try:
            if interaction.user.id not in Func.select_role_panel:
                await interaction.response.send_message("役職パネルを選択してください。", ephemeral=True)
                return
            await interaction.response.defer(ephemeral=True)
            message_id = Func.select_role_panel[interaction.user.id]
            message = await interaction.channel.fetch_message(message_id)
            if message == None:
                await interaction.followup.send("役職パネルのメッセージが見つかりません。")
                return
            embed = message.embeds[0]
            if embed == None:
                await interaction.followup.send("役職パネルが見つかりません。")
                return
            if self.type == "タイトル":
                embed.title = self.input.value
            elif self.type == "説明":
                embed.description = self.input.value
            await message.edit(embed=embed)
            await interaction.followup.send("役職パネルを更新しました。", ephemeral=True)
        except Exception:
            traceback.print_exc()

async def send_role_panel_edit_modal(interaction: discord.Interaction, type: str, value: str = None):
    modal = RolePanelLibRolePanelEditModal(interaction.client, type, value)
    await interaction.response.send_modal(modal)

class RolePanelLibRolePanelCommandCog(commands.Cog):
    def __init__(self, bot: commands.Bot):
        self.bot: commands.Bot = bot
        
    # テストコマンド
    @commands.Cog.listener()
    async def on_ready(self):
        print(f'load command cog: {self.__class__.__name__}')
        super().__init__()  # this is now required in this context.

    async def role_panel_cmd_autocomplete(self,
        interaction: discord.Interaction,
        type: str,
    ) -> List[app_commands.Choice[str]]:
        types = ['許可', '禁止', '特殊', '付与専用', '取外専用']
        return [
            app_commands.Choice(name=choice, value=choice)
            for choice in types if type.lower() in choice.lower()
        ]

    @app_commands.command(name="役職パネル", description="役職付与パネルを生成します。")
    @app_commands.checks.has_permissions(administrator=True)
    @app_commands.autocomplete(single=role_panel_cmd_autocomplete)
    @app_commands.describe(single="ロールの重複を許可するか(必須)")
    @app_commands.describe(title="パネルのタイトル(必須)")
    @app_commands.describe(role_a="役職A(必須)")
    @app_commands.describe(description="パネルの説明")
    @app_commands.describe(role_b="役職B")
    @app_commands.describe(role_c="役職C")
    @app_commands.describe(role_d="役職D")
    @app_commands.describe(role_e="役職E")
    @app_commands.describe(role_f="役職F")
    @app_commands.describe(role_g="役職G")
    @app_commands.describe(role_h="役職H")
    @app_commands.describe(role_i="役職I")
    @app_commands.describe(role_j="役職J")
    @app_commands.describe(role_k="役職K")
    @app_commands.describe(role_l="役職L")
    @app_commands.describe(role_m="役職M")
    @app_commands.describe(role_n="役職N")
    @app_commands.describe(role_o="役職O")
    @app_commands.describe(role_p="役職P")
    @app_commands.describe(role_q="役職Q")
    @app_commands.describe(role_r="役職R")
    @app_commands.describe(role_s="役職S")
    @app_commands.describe(role_t="役職T")
    @app_commands.describe(role_u="役職U")
    @app_commands.describe(role_v="役職V")
    async def role_panel_v2(
        self, 
        interaction: discord.Interaction,
        title: str,
        single: str,
        role_a: discord.Role,
        description: str = None,
        role_b: discord.Role = None,
        role_c: discord.Role = None,
        role_d: discord.Role = None,
        role_e: discord.Role = None,
        role_f: discord.Role = None,
        role_g: discord.Role = None,
        role_h: discord.Role = None,
        role_i: discord.Role = None,
        role_j: discord.Role = None,
        role_k: discord.Role = None,
        role_l: discord.Role = None,
        role_m: discord.Role = None,
        role_n: discord.Role = None,
        role_o: discord.Role = None,
        role_p: discord.Role = None,
        role_q: discord.Role = None,
        role_r: discord.Role = None,
        role_s: discord.Role = None,
        role_t: discord.Role = None,
        role_u: discord.Role = None,
        role_v: discord.Role = None
        ):
        try:
            await Func.send_role_panel_embed(interaction, self.bot, title, single, role_a, description, role_b, role_c, role_d, role_e, role_f, role_g, role_h, role_i, role_j, role_k, role_l, role_m, role_n, role_o, role_p, role_q, role_r, role_s, role_t, role_u, role_v)
        except Exception:
            traceback.print_exc()
            
    @app_commands.command(name="役職パネルロール追加", description="役職付与パネルにロールを追加します。")
    @app_commands.checks.has_permissions(administrator=True)
    @app_commands.describe(role="追加するロール(必須)")
    @app_commands.describe(emoji="絵文字")
    async def add_role_role_panel(self, interaction: discord.Interaction, role: discord.Role, emoji: str = None):
        if interaction.user.id not in Func.select_role_panel:
            await interaction.response.send_message("役職パネルを選択してください。", ephemeral=True)
            return
        await interaction.response.defer(ephemeral=True)
        message_id = Func.select_role_panel[interaction.user.id]
        message = await interaction.channel.fetch_message(message_id)
        await Func.add_role_role_panel(interaction, message, role, emoji)
    
    @app_commands.command(name="役職パネルロール削除", description="役職付与パネルからロールを削除します。")
    @app_commands.checks.has_permissions(administrator=True)
    @app_commands.describe(role="削除するロール(必須)")
    async def remove_role_role_panel(self, interaction: discord.Interaction, role: discord.Role):
        if interaction.user.id not in Func.select_role_panel:
            await interaction.response.send_message("役職パネルを選択してください。", ephemeral=True)
            return
        await interaction.response.defer(ephemeral=True)
        message_id = Func.select_role_panel[interaction.user.id]
        message = await interaction.channel.fetch_message(message_id)
        await Func.remove_role_role_panel(interaction, message, role)

    async def role_panel_edit_cmd_autocomplete(self,
        interaction: discord.Interaction,
        type: str,
    ) -> List[app_commands.Choice[str]]:
        types = ['タイトル', '説明', '重複許可']
        return [
            app_commands.Choice(name=choice, value=choice)
            for choice in types if type.lower() in choice.lower()
        ]    

    @app_commands.command(name="役職パネル編集", description="役職付与パネルを編集します。")
    @app_commands.checks.has_permissions(administrator=True)
    @app_commands.describe(type="編集する項目(必須)")
    @app_commands.autocomplete(type=role_panel_edit_cmd_autocomplete)
    async def role_panel_edit(
        self,
        interaction: discord.Interaction,
        type: str,
        value: str = None
    ):
        try:
            if interaction.user.id not in Func.select_role_panel:
                await interaction.response.send_message("役職パネルを選択してください。", ephemeral=True)
                return
            await interaction.response.defer(ephemeral=True)
            message_id = Func.select_role_panel[interaction.user.id]
            message = await interaction.channel.fetch_message(message_id)
            if message == None:
                await interaction.followup.send("役職パネルのメッセージが見つかりません。", ephemeral=True)
                return
            embed = message.embeds[0]
            if embed == None:
                await interaction.followup.send("役職パネルが見つかりません。", ephemeral=True)
                return
            if type == "タイトル":
                if value == None:
                    await interaction.followup.send("タイトルを入力してください。", ephemeral=True)
                    return
                if embed.title == value:
                    await interaction.followup.send("タイトルは変更されていません。", ephemeral=True)
                    return
                embed.title = value
            elif type == "説明":
                if value == None:
                    await interaction.followup.send("説明を入力してください。", ephemeral=True)
                    return
                if embed.description == value:
                    await interaction.followup.send("説明は変更されていません。", ephemeral=True)
                    return
                value = value.replace("/n", "\n") if "/n" in value else value
                embed.description = value
            elif type == "重複許可":
                select: discord.ui.Select = discord.ui.Select(
                    placeholder="重複許可を選択してください。",
                    custom_id=f"{Func.get_custom_id()}edit_select"
                )
                select.add_option(label="許可", value="multiple")
                select.add_option(label="禁止", value="single")
                select.add_option(label="特殊", value="special")
                select.add_option(label="付与専用", value="add_only")
                select.add_option(label="取り外し専用", value="remove_only")
                view: discord.ui.View = discord.ui.View()
                view.add_item(select)
                await interaction.followup.send(
                    "重複許可を選択してください。",
                    view=view
                )
                return
            await message.edit(embed=embed)
            await interaction.followup.send("役職パネルを更新しました。", ephemeral=True)
        except Exception:
            traceback.print_exc()
            await interaction.followup.send("役職パネルの編集に失敗しました。", ephemeral=True)
    

async def setup(bot: commands.Bot):
    await bot.add_cog(RolePanelLibRolePanelCommandCog(bot))


import discord
from discord.ext import commands
import traceback

CUSTOM_ID: str = "role_panel_"
BUTTON_COLOR: discord.ButtonStyle = discord.ButtonStyle.primary

def set_options(custom_id: str = None, button_color: discord.ButtonStyle = None):
    if custom_id:
        global CUSTOM_ID
        CUSTOM_ID = custom_id
    if button_color:
        global BUTTON_COLOR
        BUTTON_COLOR = button_color

def get_button_style() -> discord.ButtonStyle:
    global BUTTON_COLOR
    if BUTTON_COLOR == None:
        BUTTON_COLOR = discord.ButtonStyle.primary
    return BUTTON_COLOR

def set_custom_id(custom_id: str):
    if custom_id == None:
        print("custom id id not none")
        return
    global CUSTOM_ID
    CUSTOM_ID = custom_id

def get_custom_id() -> str:
    global CUSTOM_ID
    if CUSTOM_ID == None:
        CUSTOM_ID = "role_panel_"
    return CUSTOM_ID

async def send_role_panel_embed(
    interaction: discord.Interaction,
    bot: commands.bot, 
    title: str,
    single: str, 
    role_a: discord.Role, 
    description: str = None, 
    role_b: discord.Role = None, 
    role_c: discord.Role = None, 
    role_d: discord.Role = None, 
    role_e: discord.Role = None, 
    role_f: discord.Role = None, 
    role_g: discord.Role = None, 
    role_h: discord.Role = None, 
    role_i: discord.Role = None, 
    role_j: discord.Role = None, 
    role_k: discord.Role = None, 
    role_l: discord.Role = None, 
    role_m: discord.Role = None, 
    role_n: discord.Role = None, 
    role_o: discord.Role = None, 
    role_p: discord.Role = None, 
    role_q: discord.Role = None, 
    role_r: discord.Role = None, 
    role_s: discord.Role = None, 
    role_t: discord.Role = None, 
    role_u: discord.Role = None, 
    role_v: discord.Role = None
    ):
    await interaction.response.defer()
    embed: discord.Embed = discord.Embed(title=title, description=description, color=0x00bfff)
    embed.set_footer(text=bot.user.name, icon_url=bot.user.avatar.url)
    view: discord.ui.View = discord.ui.View()
    type_text: str = None
    if single == "許可":
        type_text = "multiple"
    elif single == "禁止":
        type_text = "single"
    elif single == "特殊":
        type_text = "special"
    elif single == "取り外し専用":
        type_text = "remove_only"
    elif single == "付与専用":
        type_text = "add_only"
    else:
        await interaction.followup.send(content="不正な役職パネルです", ephemeral=True)
        return
    text: str = f"🇦:{role_a.mention}"
    try:
        view.add_item(discord.ui.Button(emoji="🇦", custom_id=f"{get_custom_id()}🇦_type_{type_text}", style=get_button_style()))
        if role_b is not None:
            text += f"\n🇧:{role_b.mention}"
            view.add_item(discord.ui.Button(emoji="🇧", custom_id=f"{get_custom_id()}🇧_type_{type_text}", style=get_button_style()))
        if role_c is not None:
            text += f"\n🇨:{role_c.mention}"
            view.add_item(discord.ui.Button(emoji="🇨", custom_id=f"{get_custom_id()}🇨_type_{type_text}", style=get_button_style()))
        if role_d is not None:
            text += f"\n🇩:{role_d.mention}"
            view.add_item(discord.ui.Button(emoji="🇩", custom_id=f"{get_custom_id()}🇩_type_{type_text}", style=get_button_style()))
        if role_e is not None:
            text += f"\n🇪:{role_e.mention}"
            view.add_item(discord.ui.Button(emoji="🇪", custom_id=f"{get_custom_id()}🇪_type_{type_text}", style=get_button_style()))
        if role_f is not None:
            text += f"\n🇫:{role_f.mention}"
            view.add_item(discord.ui.Button(emoji="🇫", custom_id=f"{get_custom_id()}🇫_type_{type_text}", style=get_button_style()))
        if role_g is not None:
            text += f"\n🇬:{role_g.mention}"
            view.add_item(discord.ui.Button(emoji="🇬", custom_id=f"{get_custom_id()}🇬_type_{type_text}", style=get_button_style()))
        if role_h is not None:
            text += f"\n🇭:{role_h.mention}"
            view.add_item(discord.ui.Button(emoji="🇭", custom_id=f"{get_custom_id()}🇭_type_{type_text}", style=get_button_style()))
        if role_i is not None:
            text += f"\n🇮:{role_i.mention}"
            view.add_item(discord.ui.Button(emoji="🇮", custom_id=f"{get_custom_id()}🇮_type_{type_text}", style=get_button_style()))
        if role_j is not None:
            text += f"\n🇯:{role_j.mention}"
            view.add_item(discord.ui.Button(emoji="🇯", custom_id=f"{get_custom_id()}🇯_type_{type_text}", style=get_button_style()))
        if role_k is not None:
            text += f"\n🇰:{role_k.mention}"
            view.add_item(discord.ui.Button(emoji="🇰", custom_id=f"{get_custom_id()}🇰_type_{type_text}", style=get_button_style()))
        if role_l is not None:
            text += f"\n🇱:{role_l.mention}"
            view.add_item(discord.ui.Button(emoji="🇱", custom_id=f"{get_custom_id()}🇱_type_{type_text}", style=get_button_style()))
        if role_m is not None:
            text += f"\n🇲:{role_m.mention}"
            view.add_item(discord.ui.Button(emoji="🇲", custom_id=f"{get_custom_id()}🇲_type_{type_text}", style=get_button_style()))
        if role_n is not None:
            text += f"\n🇳:{role_n.mention}"
            view.add_item(discord.ui.Button(emoji="🇳", custom_id=f"{get_custom_id()}🇳_type_{type_text}", style=get_button_style()))
        if role_o is not None:
            text += f"\n🇴:{role_o.mention}"
            view.add_item(discord.ui.Button(emoji="🇴", custom_id=f"{get_custom_id()}🇴_type_{type_text}", style=get_button_style()))
        if role_p is not None:
            text += f"\n🇵:{role_p.mention}"
            view.add_item(discord.ui.Button(emoji="🇵", custom_id=f"{get_custom_id()}🇵_type_{type_text}", style=get_button_style()))
        if role_q is not None:
            text += f"\n🇶:{role_q.mention}"
            view.add_item(discord.ui.Button(emoji="🇶", custom_id=f"{get_custom_id()}🇶_type_{type_text}", style=get_button_style()))
        if role_r is not None:
            text += f"\n🇷:{role_r.mention}"
            view.add_item(discord.ui.Button(emoji="🇷", custom_id=f"{get_custom_id()}🇷_type_{type_text}", style=get_button_style()))
        if role_s is not None:
            text += f"\n🇸:{role_s.mention}"
            view.add_item(discord.ui.Button(emoji="🇸", custom_id=f"{get_custom_id()}🇸_type_{type_text}", style=get_button_style()))
        if role_t is not None:
            text += f"\n🇹:{role_t.mention}"
            view.add_item(discord.ui.Button(emoji="🇹", custom_id=f"{get_custom_id()}🇹_type_{type_text}", style=get_button_style()))
        if role_u is not None:
            text += f"\n🇺:{role_u.mention}"
            view.add_item(discord.ui.Button(emoji="🇺", custom_id=f"{get_custom_id()}🇺_type_{type_text}", style=get_button_style()))
        if role_v is not None:
            text += f"\n🇻:{role_v.mention}"
            view.add_item(discord.ui.Button(emoji="🇻", custom_id=f"{get_custom_id()}🇻_type_{type_text}", style=get_button_style()))
        embed.add_field(name="役職パネル", value=text, inline=False)
        # embed.add_field(name="重複許可", value=single, inline=False)
        msg: discord.Message = await interaction.followup.send(content="役職パネルを投稿します")
        await interaction.channel.send(embed=embed, view=view)
        await msg.delete()
    except Exception:
        traceback.print_exc()  # どこでエラーが発生したか表示
        await interaction.followup.send(content="インタラクションに失敗しました。", ephemeral=True)

select_role_panel: dict = {}

def add_select_role_panel(user_id: int, message_id: int):
    global select_role_panel
    select_role_panel[user_id] = message_id

def get_select_role_panel(user_id: int) -> int:
    global select_role_panel
    return select_role_panel[user_id]

async def add_role_role_panel(interaction: discord.Interaction, message: discord.Message, role: discord.Role, emoji=None):
    try:
        if len(message.embeds) == 0:
            await interaction.followup.send("役職パネルを選択してください。", ephemeral=True)
            return
        embed: discord.Embed = message.embeds[0]
        if len(embed.fields) == 0:
            await interaction.followup.send("役職パネルを選択してください。", ephemeral=True)
            return
        text: str = embed.fields[0].value
        role_list: list[str] = text.split("\n")
        view: discord.ui.View = message.components[0]
        button: discord.ui.Button = None
        for child in view.children:
            if isinstance(child, discord.ui.Button):
                button = child
                break
        if button != None:
            if "_type_" in button.custom_id:
                type_text = button.custom_id.split("_type_")[1]
            else:
                if embed.fields[1].value == "許可":
                    type_text = "multiple"
                elif embed.fields[1].value == "禁止" or embed.fields[1].value == "重複を許可しない":
                    type_text = "single"
                elif embed.fields[1].value == "特殊":
                    type_text = "special"
                elif embed.fields[1].value == "付与専用":
                    type_text = "add_only"
                elif embed.fields[1].value == "取り外し専用":
                    type_text = "remove_only"
        if type_text == None:
            await interaction.followup.send(content="役職パネルが不正です。", ephemeral=True)
            return
        view = discord.ui.View()
        for role_temp in role_list:
            # :任意の文字:部分を取得
            role_emoji = role_temp.split(":")[0]
            role_text = role_temp.split(":")[1]
            view.add_item(discord.ui.Button(emoji=role_emoji, custom_id=f"{get_custom_id()}{role_emoji}_type_{type_text}", style=get_button_style()))
            if role_text == role.mention:
                await interaction.followup.send("その役職はすでに登録されています。", ephemeral=True)
                return
            reactions = ['🇦', '🇧', '🇨', '🇩', '🇪', '🇫', '🇬', '🇭', '🇮', '🇯', '🇰', '🇱', '🇲', '🇳', '🇴', '🇵', '🇶', '🇷', '🇸', '🇹', '🇺', '🇻', '🇼', '🇽', '🇾', '🇿'] 
        final_emoji = None
        for emoji in reactions:
            if emoji not in text:
                final_emoji = emoji
                break
        if final_emoji is None:
            await interaction.followup.send("役職パネルの役職がいっぱいです。", ephemeral=True)
            return
        text += f"\n{final_emoji}:{role.mention}"
        embed.set_field_at(0, name="役職パネル", value=text, inline=False)
        view.add_item(discord.ui.Button(emoji=final_emoji, custom_id=f"{get_custom_id()}{final_emoji}_type_{type_text}", style=get_button_style()))
        await message.edit(embed=embed, view=view)
        await interaction.followup.send("役職パネルを更新しました。", ephemeral=True)
    except Exception:
        traceback.print_exc()

async def remove_role_role_panel(interaction: discord.Interaction, message: discord.Message, role: discord.Role):
    try:
        if len(message.embeds) == 0:
            await interaction.followup.send("役職パネルを選択してください。")
            return
        embed: discord.Embed = message.embeds[0]
        if len(embed.fields) == 0:
            await interaction.followup.send("役職パネルを選択してください。")
            return
        text: str = ""
        role_list: list[str] = embed.fields[0].value.split("\n")
        view: discord.ui.View = message.components[0]
        button: discord.ui.Button = None
        for child in view.children:
            if isinstance(child, discord.ui.Button):
                button = child
                break
        if button != None:
            if "_type_" in button.custom_id:
                type_text = button.custom_id.split("_type_")[1]
            else:
                if embed.fields[1].value == "許可":
                    type_text = "multiple"
                elif embed.fields[1].value == "禁止" or embed.fields[1].value == "重複を許可しない":
                    type_text = "single"
                elif embed.fields[1].value == "特殊":
                    type_text = "special"
                elif embed.fields[1].value == "付与専用":
                    type_text = "add_only"
                elif embed.fields[1].value == "取り外し専用":
                    type_text = "remove_only"
        if type_text == None:
            await interaction.followup.send(content="役職パネルが不正です。", ephemeral=True)
            return
        view = discord.ui.View()
        for role_temp in role_list:
            # :任意の文字:部分を取得
            role_emoji = role_temp.split(":")[0]
            role_text = role_temp.split(":")[1]
            try:
                role = interaction.guild.get_role(int(role_text.strip("<@&>")))
            except:
                role = None
            if role is None:
                continue
            if text != "":
                text += "\n"
            text += f"{role_emoji}:{role.mention}"
            view.add_item(discord.ui.Button(emoji=role_emoji, custom_id=f"{get_custom_id()}{role_emoji}_type_{type_text}", style=get_button_style()))
        embed.set_field_at(0, name="役職パネル", value=text, inline=False)
        await message.edit(embed=embed, view=view)
        await interaction.followup.send("役職パネルを更新しました。")
    except Exception:
        traceback.print_exc()

async def fix_select_role_panel(interaction: discord.Interaction, message: discord.Message):
    try:
        if len(message.embeds) == 0:
            await interaction.followup.send("役職パネルを選択してください。", ephemeral=True)
            return
        embed: discord.Embed = message.embeds[0]
        if len(embed.fields) == 0:
            await interaction.followup.send("役職パネルを選択してください。", ephemeral=True)
            return
        text: str = embed.fields[0].value
        role_list: list[str] = text.split("\n")
        temp_view: discord.ui.View = message.components[0]
        type_text: str = ""
        for i in range(len(temp_view.children)):
            print(temp_view.children[i])
            if isinstance(temp_view.children[i], discord.ui.Button):
                button: discord.ui.Button = temp_view.children[i]
                if button.custom_id.startswith(get_custom_id()):
                    emoji = button.custom_id.split("_")[2]
                    type_text_temp = button.custom_id.split("_type_")[1] if "_type_" in button.custom_id else ""
                    if type_text_temp != "" and type_text == "":
                        type_text = type_text_temp
        if type_text == "" and len(embed.fields) >= 2:
            if embed.fields[1].name == "重複許可":
                if embed.fields[1].value == "許可":
                    type_text = "multiple"
                elif embed.fields[1].value == "禁止":
                    type_text = "single"
                elif embed.fields[1].value == "特殊":
                    type_text = "special"
                elif embed.fields[1].value == "取り外し専用":
                    type_text = "remove_only"
                elif embed.fields[1].value == "付与専用":
                    type_text = "add_only"
        if type_text == "":
            await interaction.followup.send(content="不正な役職パネルです。", ephemeral=True)
            return
        title: str = embed.title
        description: str = embed.description
        color = embed.color
        footer_text: str = embed.footer.text
        footer_icon: str = embed.footer.icon_url
        
        embed = discord.Embed(title=title, description=description, color=color)
        embed.set_footer(text=footer_text, icon_url=footer_icon)

        view: discord.ui.View = discord.ui.View()
        text = ""
        for role_temp in role_list:
            # :任意の文字:部分を取得
            role_emoji = role_temp.split(":")[0]
            role_text = role_temp.split(":")[1]
            try:
                role = interaction.guild.get_role(int(role_text.strip("<@&>")))
            except:
                role = None
            if role is None:
                continue
            if text != "":
                text += "\n"
            text += f"{role_emoji}:{role.mention}"
            view.add_item(discord.ui.Button(emoji=role_emoji, custom_id=f"{get_custom_id()}{role_emoji}_type_{type_text}", style=get_button_style()))
        embed.add_field(name="役職パネル", value=text)
        await message.edit(embed=embed, view=view)
        await interaction.followup.send("役職パネルを更新しました。", ephemeral=True)
    except Exception:
        traceback.print_exc()


from setuptools import setup
import re

def derive_version() -> str:
    version = ''
    with open('discord_py_role_panel_lib/__init__.py', encoding='utf-8') as f:
        version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

    if not version:
        raise RuntimeError('version is not set')

    if version.endswith(('a', 'b', 'rc')):
        # append version identifier based on commit count
        try:
            import subprocess

            p = subprocess.Popen(['git', 'rev-list', '--count', 'HEAD'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            out, err = p.communicate()
            if out:
                version += out.decode('utf-8').strip()
            p = subprocess.Popen(['git', 'rev-parse', '--short', 'HEAD'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            out, err = p.communicate()
            if out:
                version += '+g' + out.decode('utf-8').strip()
        except Exception:
            pass

    return version

NAME = 'discord-py-role-panel-lib'

PACKAGES = [
    'discord_py_role_panel_lib',
    'discord_py_role_panel_lib.utils',
    'discord_py_role_panel_lib.cm',
    'discord_py_role_panel_lib.cmds',
    'discord_py_role_panel_lib.events',
    'discord_py_role_panel_lib.events.buttons',
    'discord_py_role_panel_lib.events.components'
]

setup(
    name=NAME,
    version=derive_version(),
    packages=PACKAGES,
    install_requires=[
        'discord.py>=2.5.2',
        # Add other dependencies here
    ],
    author='hashimotok',
    author_email='contact@hashimotok.dev',
    url=f'https://github.com/hashimotok-ecsv/{NAME}',
    download_url=f'https://github.com/hashimotok-ecsv/{NAME}',
    python_requires=">=3.10.6",
    description='A library to assist with Discord.py development',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Operating System :: OS Independent',
    ],
    keywords='discord.py role panel library',
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'discord-py-role-panel-lib=discord_py_role_panel_lib.__main__:main',
        ],
    },
    project_urls={
        'Documentation': f'https://{NAME}.readthedocs.io/',
        'Source': f'https://github.com/hashimotok-ecsv/{NAME}',
        'Tracker': f'https://github.com/hashimotok-ecsv/{NAME}/issues',
    },
    license='MIT',
    license_files=('LICENSE',),
)


import discord
from discord.ext import commands
import traceback

class RolePanelLibTemplateButtonEvent():
    def __init__(self, bot: commands.Bot):
        self.bot: commands.Bot = bot

    async def call(self, inter: discord.Interaction):
        try:
            custom_id: str = inter.data["custom_id"] #interaction.dataからcustom_idを取り出す
            if custom_id == "":
                pass
        except Exception:
            traceback.print_exc()


import discord
from discord.ext import commands
import traceback

from ..events.components import normal_select_event
from ..events.buttons import role_panel_button_event

from ..utils import role_panel_function as Func

class RolePanelLibButtonClickCog(commands.Cog):
    def __init__(self, bot: commands.Bot):
        self.bot: commands.Bot = bot
        self.normal_select_event = normal_select_event.RolePanelLibNormalSelectEvent(bot)
        # 可読性を向上させるために、各ボタンイベントをインスタンス化
        self.role_panel_button_event = role_panel_button_event.RolePanelLibRolePanelButtonEvent(bot)

    # コグアンロード処理
    def cog_unload(self):
        return super().cog_unload()

    @commands.Cog.listener()
    async def on_ready(self):
        print(f'load event cog: {self.__class__.__name__}')
        super().__init__()  # this is now required in this context.

    @commands.Cog.listener()
    async def on_interaction(self, inter:discord.Interaction):
        try:
            print(inter.data['component_type']) # TODO: 後で消す
            if inter.data['component_type'] == 2:
                await self.on_button_click(inter)
            elif inter.data['component_type'] == 3:
                await self.normal_select_event.call(inter)
            elif inter.data['component_type'] == 5:
                # await self.user_select_event.call(inter)
                pass
        except KeyError:
            pass

    async def on_button_click(self, inter: discord.Interaction): 
        custom_id: str = inter.data["custom_id"]
        print(f"button: {custom_id}")
        try:
            if custom_id.startswith(Func.get_custom_id()):
                await self.role_panel_button_event.call(inter)
        except Exception:
            traceback.print_exc()
            await inter.response.send_message(content="エラーが発生しました。", ephemeral=True)

async def setup(bot: commands.Bot):
    await bot.add_cog(RolePanelLibButtonClickCog(bot))